package demo.moazs;

import at.gv.util.MiscUtil;
import at.gv.util.client.mzs.MOAZSClient;
import at.gv.util.config.EgovUtilConfiguration;
import at.gv.util.config.EgovUtilPropertiesConfiguration;
import at.gv.util.xsd.mzs.DeliveryRequestType;
import at.gv.util.xsd.mzs.DeliveryRequestType.MetaData;
import at.gv.util.xsd.mzs.DeliveryRequestType.MetaData.MailBody;
import at.gv.util.xsd.mzs.DeliveryRequestType.Payload;
import at.gv.util.xsd.mzs.DeliveryRequestType.Payload.BinaryDocument;
import at.gv.util.xsd.mzs.DeliveryRequestType.Sender;
import at.gv.util.xsd.mzs.DeliveryResponseType;
import at.gv.util.xsd.mzs.persondata.InternetAddressType;
import at.gv.util.xsd.mzs.persondata.ObjectFactory;
import at.gv.util.xsd.mzs.persondata.PersonDataType;
import at.gv.util.xsd.mzs.persondata.PersonNameType;
import at.gv.util.xsd.mzs.persondata.PersonNameType.FamilyName;
import at.gv.util.xsd.mzs.persondata.PhysicalPersonType;

public class App2MZS {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		
		// load configuration
		EgovUtilConfiguration propsConfig = new EgovUtilPropertiesConfiguration("sample-conf/moazs.properties");

		// prepare request
		DeliveryRequestType request = new DeliveryRequestType();
		
		// sync delivery
		request.setSync(Boolean.TRUE);
		
		// uncomment this, if the central lookup service (Zustellkopf) has already been contacted
		//Server server = new Server();
		//server.setZUSEUrlID("http://zuseurl.com/DeliveryRequest");
		//server.setX509(certificate);
		//request.setServer(server);
		ObjectFactory of = new ObjectFactory();
		Sender sender = new Sender();
		sender.setProfileID("123456");
		// uncomment this, if the sender is set manually
		
		//CorporateBodyType cbt = of.createCorporateBodyType();
		//cbt.setFullName("Testbehoerde");
		//cbt.setOrganization("Testorganization");
		//sender.setPerson(of.createCorporateBody(cbt));
		request.setSender(sender);
		
		// set receiver data
		PhysicalPersonType ppt = of.createPhysicalPersonType();
		PersonNameType pnt = new PersonNameType();
		pnt.setGivenName("Arne");
		FamilyName fnt = new FamilyName();
		fnt.setValue("Tauber");
		pnt.setFamilyName(fnt);
		ppt.setName(pnt);
		ppt.setDateOfBirth(MiscUtil.getXMLGregorianCalendar("1979-08-21"));
		PersonDataType pdt = new PersonDataType();
		pdt.setPerson(of.createPhysicalPerson(ppt));
		InternetAddressType iat = new InternetAddressType();
		iat.setAddress("arne.tauber@egiz.gv.at");
		pdt.getAddress().add(of.createInternetAddress(iat));
		request.setReceiver(pdt);
		
		// set meta-data
		MetaData md = new MetaData();
		md.setAppDeliveryID("ABCDEF");
		md.setDeliveryQuality("RSa");
		md.setRequiresEncryption(false);
		// uncomment in case of private delivery
		//md.setPrivateDelivery(Boolean.FALSE);
		//md.setDocumentClass("bill");
		// uncomment in case of dual delivery
		//md.setDualDelivery(...);
		
		// optionally use a custom mailbody
		MailBody mb = new MailBody();
		mb.setMimeType("text/plain");
		mb.setContent("This delivery has been sent by authority Testauthority.".getBytes());
		md.setMailBody(mb);
		request.setMetaData(md);
		
		// set payload
		Payload payload = new Payload();
		BinaryDocument bd = new BinaryDocument();
		bd.setFileName("Anhang.txt");
		bd.setMIMEType("text/plain");
		bd.setBase64Content("Ich bin ein Anhang.".getBytes());
		payload.setBinaryDocument(bd);
		request.getPayload().add(payload);

		// send delivery request
		MOAZSClient client = new MOAZSClient(propsConfig);
		DeliveryResponseType response = client.sendMOAZSRequest(request, "http://localhost:8080/moa-zs/services/DeliveryRequest");
		System.out.println("Success: " + response.getSuccess() != null);
	}

}
