package demo.zusekopf;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.codec.binary.Base64;

import at.gv.zustellung.zkopf.ZuseKopfConstants;
import at.gv.zustellung.zkopf.client.PostalAddress;
import at.gv.zustellung.zkopf.client.StdQueryClient;
import at.gv.zustellung.zkopf.config.PropertiesZuseKopfConfiguration;
import at.gv.zustellung.zkopf.query.DefaultStandardQuery;
import at.gv.zustellung.zkopf.xsd.zkopf.StdAnswerType;
import at.gv.zustellung.zkopf.xsd.zkopf.ZuseType;


public class DemoStandardQuery {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		
		// URL of Zustellkopf Standard Query Service
		String zkopfServiceURL = "https://zustellung.egiz.gv.at/zkopf-test/Query?";
		
		// load zusekopf configuration
		PropertiesZuseKopfConfiguration propsConfig = new PropertiesZuseKopfConfiguration("sample-conf/zusekopf.properties");
		
		// create client
		StdQueryClient zkCore = new StdQueryClient(propsConfig);
		
		// make a query based on ZustellbPK
		DefaultStandardQuery query = new DefaultStandardQuery("H7GgqGbN55YDrI/Xma3qdZmDZAmfY6oDs9ZvMXRU720JKwwrCOvFPazCZ+qCjx0MUOKf8Sr9HfPopK+tdpxqSEBji2p//X/8sXZwHN/kg7Eohry25SigvRtcCraspRjDAeN2nLeWmNrwGqRLCDyFNK+yacvD/N5/oasN/yBRTI8=", null);
		StdAnswerType answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query using special characters		
		query = new DefaultStandardQuery("Max", "Mueller", null, "max@mueller.at", null, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
	  // make a query based on a physical person and e-mail address (nonRSa)		
		query = new DefaultStandardQuery("Max", "Mustermann", null, "max@mustermann.at", null, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a physical person and e-mail address (RSa)		
		query = new DefaultStandardQuery("Max", "Mustermann", "1980-12-31", "max@mustermann.at", null, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a physical person and telephone number		
		query = new DefaultStandardQuery("Max", "Mustermann", null, null, "+4311234567890", false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a corporate body
		query = new DefaultStandardQuery("123456a", ZuseKopfConstants.SOURCEPIN_TYPE_STD_QUERY_XFN, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));

		// make a private query based on a physical person (Privatzustellung)		
		query = new DefaultStandardQuery("Max", "Mustermann", null, "max@mustermann.at", null, true, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a physical person and postal address
		PostalAddress postalAddress = new PostalAddress("Mustergasse 1", "1010", "Wien", "AT");
		query = new DefaultStandardQuery("Arne", "Tauber", null, postalAddress, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a corporate body and e-mail address
		query = new DefaultStandardQuery("Mustergemeinde", "muster@gemeinde.at", null, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a corporate body and telephone address
		query = new DefaultStandardQuery("Mustergemeinde", null, "+4311234567890", false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on a corporate body and postal address
		postalAddress = new PostalAddress("Mustergasse 1", "1010", "Wien", "AT");
		query = new DefaultStandardQuery("Mustergemeinde", postalAddress, false, null);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));

		// make a query based on a corporate body and postal address (Privatzustellung) and customer number
		postalAddress = new PostalAddress("Mustergasse 1", "1010", "Wien", "AT");
		query = new DefaultStandardQuery("Mustergemeinde", postalAddress, true, "123456");
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));
		
		// make a query based on ERV code
		Map<String, String> params = new HashMap<String, String>();
		params.put(ZuseKopfConstants.HTTP_QUERY_PARAM_ERV_CODE, "P-12345");
		query = new DefaultStandardQuery(params);
		answer = zkCore.stdQuery(zkopfServiceURL, query);
		System.out.println(DemoStandardQuery.toString(answer));		
		
	}

	private static String toString(StdAnswerType answer) {
		StringBuffer buffer = new StringBuffer();
		if (answer.getSuccess() != null) {
			buffer.append("Zkopf-response: Success\r\n");
			if (answer.getSuccess().getGvZbPK() != null) {
				buffer.append("  gvZbPK: " + answer.getSuccess().getGvZbPK());
			}
			for (ZuseType server : answer.getSuccess().getServer()) {
				buffer.append("\r\n\r\n");
				buffer.append("    Server WS-URL: " + server.getZUSEUrlID() + "\r\n");
				if (server.getEdID() != null) {
					buffer.append("             edID: " + server.getEdID() + "\r\n");
				}
				buffer.append("       MIME-types: " + server.getMIMETypes() + "\r\n");
				if (server.getX509() != null) {
					buffer.append("  X509Certificate: " + Base64.encodeBase64String(server.getX509()));
				}
			}
		} else {
			buffer.append("Error\r\n\r\n");
			buffer.append(" Code: " + answer.getError().intValue());
		}
		return buffer.toString();
	}
	
	
}
