/*
 * Copyright 2011 E-Government Innovation Center (EGIZ)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package demo.zusemsg;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Calendar;

import org.apache.commons.lang.RandomStringUtils;

import at.gv.util.MiscUtil;
import at.gv.zustellung.zusemsg.config.PropertiesZuseMsgConfiguration;
import at.gv.zustellung.zusemsg.config.ZuseMsgConfiguration;
import at.gv.zustellung.zusemsg.wsdl.client.App2ZuseClient;
import at.gv.zustellung.zusemsg.wsdl.client.attachments.BinaryAttachment;
import at.gv.zustellung.zusemsg.wsdl.client.attachments.MailBody;
import at.gv.zustellung.zusemsg.xsd.zuse.DeliveryRequestStatusType;
import at.gv.zustellung.zusemsg.xsd.zuse.DeliveryRequestType;
import at.gv.zustellung.zusemsg.xsd.zuse.DeliveryRequestType.MetaData;
import at.gv.zustellung.zusemsg.xsd.zuse.DeliveryRequestType.NotificationAddress;
import at.gv.zustellung.zusemsg.xsd.zuse.DeliveryRequestType.NotificationAddress.WebserviceURL;
import at.gv.zustellung.zusemsg.xsd.zuse.Receiver;
import at.gv.zustellung.zusemsg.xsd.zuse.Sender;
import at.gv.zustellung.zusemsg.xsd.zuse_persondata.CorporateBodyType;
import at.gv.zustellung.zusemsg.xsd.zuse_persondata.InternetAddressType;
import at.gv.zustellung.zusemsg.xsd.zuse_persondata.ObjectFactory;
import at.gv.zustellung.zusemsg.xsd.zuse_persondata.PersonNameType;
import at.gv.zustellung.zusemsg.xsd.zuse_persondata.PersonNameType.FamilyName;
import at.gv.zustellung.zusemsg.xsd.zuse_persondata.PhysicalPersonType;

public class DemoApp2Zuse {

	/**
	 * @param args
	 */
	@SuppressWarnings("unused")
  public static void main(String[] args) throws Exception {

		// load configuration
		ZuseMsgConfiguration config = new PropertiesZuseMsgConfiguration("sample-conf/zusemsg.properties");
		// init client
		App2ZuseClient client = new App2ZuseClient(config);
		
		// create delivery request
		DeliveryRequestType request = new DeliveryRequestType();
		
		// set identifier received by zustellkopf
		// moabox
		request.setZbPK("dtprhcHmBJUT4Z0iYzHypZWnZtn1gr952DT66H6GSDfoDFEDdWL0fCYZqCra+I+Uo/qIKfDwhHIvoyeP4RXTygVywEFrgj62nDhigwbRg4xmDcPm1z/2fv6+AefVOWKaDsPLX8yk/p45Twg0hqMxd3h37p8hrmoUfrpBDB5YO+4=");
		
		// uncomment this if conducting a private delivery 
		//request.setEdID("TBD");
		
		// set notification address
		NotificationAddress notificationAddress = new NotificationAddress();
		WebserviceURL wsu = new WebserviceURL();
		wsu.setValue("http://demo.egiz.gv.at/dumpzs/Dump");
		wsu.setAlternativeEmail("no-reply@egiz.gv.at");
		//notificationAddress.setWebserviceURL("http://apps.egiz.gv.at/zuse2app/zuse2app");
		notificationAddress.setWebserviceURL(wsu);

		// uncomment this if using an e-mail address for notifications
		//InternetAddressType iat = new InternetAddressType();
		//iat.setAddress("arne.tauber@egiz.gv.at);
		//notificationAddress.setEmail(iat);
		request.setNotificationAddress(notificationAddress);
		
		// set sender data
		Sender sender = new Sender();
		//Organisation organisation = new Organisation();
		CorporateBodyType organisation = new CorporateBodyType();
		organisation.setFullName("Test Authority");
		//sender.setOrganisation(organisation);
		sender.setCorporateBody(organisation);
		InternetAddressType adt = new InternetAddressType();
		adt.setAddress("test@authority.at");
	  sender.getAddress().add(new ObjectFactory().createAddress(adt));
		request.setSender(sender);
		
		// set receiver data
		Receiver receiver = new Receiver();
		PhysicalPersonType ppt = new PhysicalPersonType();
		PersonNameType pnt = new PersonNameType();
		FamilyName fnt = new FamilyName();
		fnt.setValue("Mustermann");
		pnt.setFamilyName(fnt);
		pnt.setGivenName("Max");
		ppt.setName(pnt);
		ppt.setDateOfBirth("1980-12-31");
		receiver.setPhysicalPerson(ppt);
		request.setReceiver(receiver);
		
		// set meta data
		MetaData metaData = new MetaData();
		metaData.setAppDeliveryID(RandomStringUtils.random(10, true, true));
		metaData.setMZSDeliveryID(RandomStringUtils.random(10, true, true));
		metaData.setDeliveryQuality("RSa"); // RSa+, nonRSa, nonRSa+, RS, RS+
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.HOUR, 1);
		metaData.setStartNotificationBefore(MiscUtil.getXMLGregorianCalendar(cal.getTime())); // does not have any effect in case of public sector deliveries
		metaData.setDeliverBefore(MiscUtil.getXMLGregorianCalendar(cal.getTime())); // does not have any effect in case of public sector deliveries
		request.setMetaData(metaData);

		// set mailbody
		MailBody mailBody = new MailBody(MailBody.TEXT_PLAIN, new ByteArrayInputStream("This is an eDelivery of a test authority.".getBytes()));
		
		// set attachment (plain text)
		BinaryAttachment attachment = new BinaryAttachment(
				"Anhang.txt", // file name
				"text/plain", // MIME type
				new ByteArrayInputStream("I am an attachment of an eDelivery".getBytes())); // attachment InputStream
		
		// set encryption certificate
		InputStream inStream = new FileInputStream("sample-conf/encryption-certificate.cer");
		X509Certificate cert = MiscUtil.readCertificate(inStream);
		inStream.close();

		// make delivery request
		DeliveryRequestStatusType response = client.sendApp2ZuseRequest(
				"https://www.moabox.at/zustellserver/services/DeliveryRequest", // URL of Zustellserver (delivery agent)
				request, // delivery request
				mailBody, // mailbox, which is sent to recipient in case of forwards
				Arrays.asList(attachment), // attachments
				false, // make ZUS container or send as MIME parts?
				null); // X.509 encryption certificate (use null if not used)
		
		// print out result
		System.out.println("Success: " + response.getSuccess() != null);
	}

}
